# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'C:/bld/qgis_1559309846308/work/python/plugins/db_manager/ui/DlgTableProperties.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_DbManagerDlgTableProperties(object):
    def setupUi(self, DbManagerDlgTableProperties):
        DbManagerDlgTableProperties.setObjectName("DbManagerDlgTableProperties")
        DbManagerDlgTableProperties.resize(885, 470)
        self.vboxlayout = QtWidgets.QVBoxLayout(DbManagerDlgTableProperties)
        self.vboxlayout.setObjectName("vboxlayout")
        self.tabs = QtWidgets.QTabWidget(DbManagerDlgTableProperties)
        self.tabs.setObjectName("tabs")
        self.tabColumns = QtWidgets.QWidget()
        self.tabColumns.setObjectName("tabColumns")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tabColumns)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.label = QtWidgets.QLabel(self.tabColumns)
        self.label.setObjectName("label")
        self.verticalLayout_4.addWidget(self.label)
        self.viewFields = QtWidgets.QTreeView(self.tabColumns)
        self.viewFields.setRootIsDecorated(False)
        self.viewFields.setObjectName("viewFields")
        self.verticalLayout_4.addWidget(self.viewFields)
        self.hboxlayout = QtWidgets.QHBoxLayout()
        self.hboxlayout.setObjectName("hboxlayout")
        self.btnAddColumn = QtWidgets.QPushButton(self.tabColumns)
        self.btnAddColumn.setObjectName("btnAddColumn")
        self.hboxlayout.addWidget(self.btnAddColumn)
        self.btnAddGeometryColumn = QtWidgets.QPushButton(self.tabColumns)
        self.btnAddGeometryColumn.setObjectName("btnAddGeometryColumn")
        self.hboxlayout.addWidget(self.btnAddGeometryColumn)
        self.btnEditColumn = QtWidgets.QPushButton(self.tabColumns)
        self.btnEditColumn.setObjectName("btnEditColumn")
        self.hboxlayout.addWidget(self.btnEditColumn)
        spacerItem = QtWidgets.QSpacerItem(13, 29, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.hboxlayout.addItem(spacerItem)
        self.btnDeleteColumn = QtWidgets.QPushButton(self.tabColumns)
        self.btnDeleteColumn.setObjectName("btnDeleteColumn")
        self.hboxlayout.addWidget(self.btnDeleteColumn)
        self.verticalLayout_4.addLayout(self.hboxlayout)
        self.tabs.addTab(self.tabColumns, "")
        self.tabConstraints = QtWidgets.QWidget()
        self.tabConstraints.setObjectName("tabConstraints")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.tabConstraints)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label_3 = QtWidgets.QLabel(self.tabConstraints)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_3.addWidget(self.label_3)
        self.viewConstraints = QtWidgets.QTreeView(self.tabConstraints)
        self.viewConstraints.setRootIsDecorated(False)
        self.viewConstraints.setObjectName("viewConstraints")
        self.verticalLayout_3.addWidget(self.viewConstraints)
        self.hboxlayout1 = QtWidgets.QHBoxLayout()
        self.hboxlayout1.setObjectName("hboxlayout1")
        self.btnAddConstraint = QtWidgets.QPushButton(self.tabConstraints)
        self.btnAddConstraint.setObjectName("btnAddConstraint")
        self.hboxlayout1.addWidget(self.btnAddConstraint)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.hboxlayout1.addItem(spacerItem1)
        self.btnDeleteConstraint = QtWidgets.QPushButton(self.tabConstraints)
        self.btnDeleteConstraint.setObjectName("btnDeleteConstraint")
        self.hboxlayout1.addWidget(self.btnDeleteConstraint)
        self.verticalLayout_3.addLayout(self.hboxlayout1)
        self.tabs.addTab(self.tabConstraints, "")
        self.tabIndexes = QtWidgets.QWidget()
        self.tabIndexes.setObjectName("tabIndexes")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.tabIndexes)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_2 = QtWidgets.QLabel(self.tabIndexes)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_2.addWidget(self.label_2)
        self.viewIndexes = QtWidgets.QTreeView(self.tabIndexes)
        self.viewIndexes.setRootIsDecorated(False)
        self.viewIndexes.setObjectName("viewIndexes")
        self.verticalLayout_2.addWidget(self.viewIndexes)
        self._3 = QtWidgets.QHBoxLayout()
        self._3.setObjectName("_3")
        self.btnAddIndex = QtWidgets.QPushButton(self.tabIndexes)
        self.btnAddIndex.setObjectName("btnAddIndex")
        self._3.addWidget(self.btnAddIndex)
        self.btnAddSpatialIndex = QtWidgets.QPushButton(self.tabIndexes)
        self.btnAddSpatialIndex.setObjectName("btnAddSpatialIndex")
        self._3.addWidget(self.btnAddSpatialIndex)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self._3.addItem(spacerItem2)
        self.btnDeleteIndex = QtWidgets.QPushButton(self.tabIndexes)
        self.btnDeleteIndex.setObjectName("btnDeleteIndex")
        self._3.addWidget(self.btnDeleteIndex)
        self.verticalLayout_2.addLayout(self._3)
        self.tabs.addTab(self.tabIndexes, "")
        self.tabComment = QtWidgets.QWidget()
        self.tabComment.setEnabled(True)
        self.tabComment.setObjectName("tabComment")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.tabComment)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_4 = QtWidgets.QLabel(self.tabComment)
        self.label_4.setObjectName("label_4")
        self.verticalLayout.addWidget(self.label_4)
        self.viewComment = QtWidgets.QLineEdit(self.tabComment)
        self.viewComment.setObjectName("viewComment")
        self.verticalLayout.addWidget(self.viewComment)
        self._4 = QtWidgets.QHBoxLayout()
        self._4.setObjectName("_4")
        self.btnAddComment = QtWidgets.QPushButton(self.tabComment)
        self.btnAddComment.setObjectName("btnAddComment")
        self._4.addWidget(self.btnAddComment)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self._4.addItem(spacerItem3)
        self.btnDeleteComment = QtWidgets.QPushButton(self.tabComment)
        self.btnDeleteComment.setObjectName("btnDeleteComment")
        self._4.addWidget(self.btnDeleteComment)
        self.verticalLayout.addLayout(self._4)
        self.tabs.addTab(self.tabComment, "")
        self.vboxlayout.addWidget(self.tabs)
        self.buttonBox = QtWidgets.QDialogButtonBox(DbManagerDlgTableProperties)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Close)
        self.buttonBox.setObjectName("buttonBox")
        self.vboxlayout.addWidget(self.buttonBox)

        self.retranslateUi(DbManagerDlgTableProperties)
        self.tabs.setCurrentIndex(0)
        self.buttonBox.accepted.connect(DbManagerDlgTableProperties.accept)
        self.buttonBox.rejected.connect(DbManagerDlgTableProperties.reject)
        QtCore.QMetaObject.connectSlotsByName(DbManagerDlgTableProperties)
        DbManagerDlgTableProperties.setTabOrder(self.tabs, self.viewFields)
        DbManagerDlgTableProperties.setTabOrder(self.viewFields, self.btnAddColumn)
        DbManagerDlgTableProperties.setTabOrder(self.btnAddColumn, self.btnAddGeometryColumn)
        DbManagerDlgTableProperties.setTabOrder(self.btnAddGeometryColumn, self.btnEditColumn)
        DbManagerDlgTableProperties.setTabOrder(self.btnEditColumn, self.btnDeleteColumn)
        DbManagerDlgTableProperties.setTabOrder(self.btnDeleteColumn, self.viewConstraints)
        DbManagerDlgTableProperties.setTabOrder(self.viewConstraints, self.btnAddConstraint)
        DbManagerDlgTableProperties.setTabOrder(self.btnAddConstraint, self.btnDeleteConstraint)
        DbManagerDlgTableProperties.setTabOrder(self.btnDeleteConstraint, self.viewIndexes)
        DbManagerDlgTableProperties.setTabOrder(self.viewIndexes, self.btnAddIndex)
        DbManagerDlgTableProperties.setTabOrder(self.btnAddIndex, self.btnAddSpatialIndex)
        DbManagerDlgTableProperties.setTabOrder(self.btnAddSpatialIndex, self.btnDeleteIndex)
        DbManagerDlgTableProperties.setTabOrder(self.btnDeleteIndex, self.buttonBox)

    def retranslateUi(self, DbManagerDlgTableProperties):
        _translate = QtCore.QCoreApplication.translate
        DbManagerDlgTableProperties.setWindowTitle(_translate("DbManagerDlgTableProperties", "Table properties"))
        self.label.setText(_translate("DbManagerDlgTableProperties", "Table columns:"))
        self.btnAddColumn.setText(_translate("DbManagerDlgTableProperties", "Add column"))
        self.btnAddGeometryColumn.setText(_translate("DbManagerDlgTableProperties", "Add geometry column"))
        self.btnEditColumn.setText(_translate("DbManagerDlgTableProperties", "Edit column"))
        self.btnDeleteColumn.setText(_translate("DbManagerDlgTableProperties", "Delete column"))
        self.tabs.setTabText(self.tabs.indexOf(self.tabColumns), _translate("DbManagerDlgTableProperties", "Columns"))
        self.label_3.setText(_translate("DbManagerDlgTableProperties", "Primary, foreign keys, unique and check constraints:"))
        self.btnAddConstraint.setText(_translate("DbManagerDlgTableProperties", "Add primary key / unique"))
        self.btnDeleteConstraint.setText(_translate("DbManagerDlgTableProperties", "Delete constraint"))
        self.tabs.setTabText(self.tabs.indexOf(self.tabConstraints), _translate("DbManagerDlgTableProperties", "Constraints"))
        self.label_2.setText(_translate("DbManagerDlgTableProperties", "Indexes defined for this table:"))
        self.btnAddIndex.setText(_translate("DbManagerDlgTableProperties", "Add index"))
        self.btnAddSpatialIndex.setText(_translate("DbManagerDlgTableProperties", "Add spatial index"))
        self.btnDeleteIndex.setText(_translate("DbManagerDlgTableProperties", "Delete index"))
        self.tabs.setTabText(self.tabs.indexOf(self.tabIndexes), _translate("DbManagerDlgTableProperties", "Indexes"))
        self.label_4.setText(_translate("DbManagerDlgTableProperties", "Comment defined for this table:"))
        self.btnAddComment.setText(_translate("DbManagerDlgTableProperties", "Add comment"))
        self.btnDeleteComment.setText(_translate("DbManagerDlgTableProperties", "Delete comment"))
        self.tabs.setTabText(self.tabs.indexOf(self.tabComment), _translate("DbManagerDlgTableProperties", "Comment"))

